// basemenu.cc
//
// Frank Hale 
// frankhale@yahoo.com
#include "sapphire.hh"

BaseMenu::BaseMenu() {

	mi = new LinkedList<BaseMenuItem>;
	
	menu.menuTitle=windowmanager_version;

	counter=0;

	font = XLoadQueryFont(wm->getDisplay(), MENU_FONT);
	if (!font) font = XLoadQueryFont(wm->getDisplay(), MENU_FONT);

	XGCValues gv;
	gv.function = GXcopy;
	gv.foreground = wm->foreground_color.pixel;
	gv.font = font->fid;
	menu_gc = XCreateGC(wm->getDisplay(), wm->getRootWindow(), GCFunction | GCForeground | GCFont, &gv);
	
  	create_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
   	 CWOverrideRedirect | CWEventMask;
  	attrib.background_pixmap = None;
  	attrib.background_pixel = wm->background_color.pixel;
  	attrib.border_pixel = wm->border_color.pixel;
  	attrib.override_redirect = True;
  	attrib.event_mask = ButtonPressMask|ButtonReleaseMask| PointerMotionMask |
    	ButtonMotionMask|ExposureMask|EnterWindowMask|LeaveWindowMask;

	menu.x = 1;
  	menu.y = 1;
	menu.x_move = 1;
  	menu.y_move = 1;
	menu.height=1;
	menu.width=1;
    
	menu.total_item_height=1;
    
    	item_width = menu.width;
	item_height = font->ascent + font->descent + 8;
	
	menu.window =
    		XCreateWindow(wm->getDisplay(), wm->getRootWindow(), menu.x, menu.y,
		  menu.width, menu.height, 1, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);

	menu.title =
    		XCreateWindow(wm->getDisplay(), menu.window, 0, 0,
		  menu.width, item_height, 0, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);
	
	menu.item_window =	
    		XCreateWindow(wm->getDisplay(), menu.window, 0, item_height - 2,
		  menu.width, menu.height-item_height, 0, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);

	
	isVisible = False;
	
	curr = new BaseMenuItem();
	
	curr = None;
	
	enterOnce = True;
	
	wm->addToMenuList(this);
}

BaseMenu::~BaseMenu() {
	LinkedListIterator<BaseMenuItem> it(mi);
	for(; it.current(); it++)
	{
		mi->remove(it.current());
	}
			
	XFreeGC(wm->getDisplay(), menu_gc);
	XFreeGC(wm->getDisplay(), select_gc);
	
	XDestroyWindow(wm->getDisplay(), menu.window);
	XDestroyWindow(wm->getDisplay(), menu.title);
	XDestroyWindow(wm->getDisplay(), menu.item_window);

	XFreePixmap(wm->getDisplay(), title_pix);
	XFreePixmap(wm->getDisplay(), menu_pix);
	XFreePixmap(wm->getDisplay(), select_pix);
	
	delete mi;
	delete curr;
}

// This updates all graphics contexts, gradient pixmaps and determines placement of all menu items.
void BaseMenu::update_all()
{
	LinkedListIterator<BaseMenuItem> it(mi);
	
	int temp_width=0;
	int temp_ascent=0;
	int temp_descent=0;
	
	unsigned int count=0;
	unsigned int char_count=0;

	menu.total_item_height=0;

	for(; it.current(); it++)
	{
		char_count = strlen(it.current()->name);

    		XTextExtents(font, it.current()->name , char_count, &direction, &ascent, 
            		&descent, &overall);		

		if (char_count < strlen(menu.menuTitle)) {
		
			char_count = strlen(menu.menuTitle);
			
			XTextExtents(font, menu.menuTitle , char_count, &direction, &ascent, 
		           &descent, &overall);		
		}
	
		if (char_count > count) {
			count = char_count; 
			temp_width  = overall.width;
			temp_ascent = overall.ascent;
			temp_descent= overall.descent;
		}
		
		menu.width = temp_width + 20;
		item_width  = temp_ascent + temp_descent + 8;
		menu.height = item_height * mi->count();
		
		it.current()->item_x = item_width/2+4;
		it.current()->item_y = item_height/2+4+menu.total_item_height;
		
		menu.total_item_height += item_height;
	}

	menu.total_item_height += item_height;
	
	XResizeWindow(wm->getDisplay(), menu.window, menu.width, menu.total_item_height - 3);
	XResizeWindow(wm->getDisplay(), menu.title, menu.width, item_height);
	XResizeWindow(wm->getDisplay(), menu.item_window, menu.width, menu.total_item_height);

	createMenuImage();
	createMenuSelectImage();
	createMenuTitleImage();
	
	curs = XDefineCursor(wm->getDisplay(), menu.window, XCreateFontCursor(wm->getDisplay(), XC_left_ptr));
	
	XGCValues gv;
	gv.tile = select_pix;
	gv.function = GXcopy;
	gv.foreground = wm->foreground_color.pixel;
	gv.background = wm->background_color.pixel;
	gv.font = font->fid;
	
	select_gc = XCreateGC(wm->getDisplay(), wm->getRootWindow(),  GCTile , &gv);

	XSetFillStyle(wm->getDisplay(), select_gc, FillTiled);
	
	gv.tile = menu_pix;
	
	back_gc = XCreateGC(wm->getDisplay(), wm->getRootWindow(),  GCTile, &gv);

	XSetFillStyle(wm->getDisplay(), back_gc, FillTiled);
}

// This function simply updates the menu without recreating the gradients and graphics contexts.
void BaseMenu::update_menu()
{
	LinkedListIterator<BaseMenuItem> it(mi);
	
	int temp_width=0;
	int temp_ascent=0;
	int temp_descent=0;
	
	unsigned int count=0;
	unsigned int char_count=0;

	menu.total_item_height=0;

	for(; it.current(); it++)
	{
		char_count = strlen(it.current()->name);

    		XTextExtents(font, it.current()->name , char_count, &direction, &ascent, 
            		&descent, &overall);		

		if (char_count < strlen(menu.menuTitle)) {
		
			char_count = strlen(menu.menuTitle);
			
			XTextExtents(font, menu.menuTitle , char_count, &direction, &ascent, 
		           &descent, &overall);		
		}
	
		if (char_count > count) {
			count = char_count; 
			temp_width  = overall.width;
			temp_ascent = overall.ascent;
			temp_descent= overall.descent;
		}
		
		menu.width = temp_width + 40;
		item_width  = temp_ascent + temp_descent + 8;
		menu.height = item_width;
		
		it.current()->item_x = item_width/2+4;
		it.current()->item_y = item_height/2+4+menu.total_item_height;
		
		menu.total_item_height += item_height;
	}

	menu.total_item_height += item_height;
	
	XResizeWindow(wm->getDisplay(), menu.window, menu.width, menu.total_item_height - 3);
	XResizeWindow(wm->getDisplay(), menu.title, menu.width, item_height);
	XResizeWindow(wm->getDisplay(), menu.item_window, menu.width, menu.total_item_height);
}

// Redraws the entire contents of the menu.
void BaseMenu::redraw()
{
	// Clear the windows to get ready to repaint them.
	XClearWindow(wm->getDisplay(), menu.title);
	XClearWindow(wm->getDisplay(), menu.item_window);

 	// We need an Iterator for our menu items. This lets us traverse through
	// the list and paint each menu item.
	LinkedListIterator<BaseMenuItem> it(mi);
	
	for(; it.current(); it++)
	{	  				
		if(strcmp(it.current()->name, "separator")==0) 
		{
			XDrawLine(wm->getDisplay(), menu.item_window, menu_gc, it.current()->item_x - 14, it.current()->item_y-5, menu.width-1, it.current()->item_y-5);	
			XDrawLine(wm->getDisplay(), menu.item_window, menu_gc, it.current()->item_x - 14, it.current()->item_y-6, menu.width-1, it.current()->item_y-6);	
		} else {
		
			XDrawString(wm->getDisplay(), menu.item_window, menu_gc, it.current()->item_x, 
			it.current()->item_y + 2, it.current()->name, strlen(it.current()->name));
		}
		
		// Draws a bounding box around each menu item. This helped me figure out the algorithm to test which item the 
		// pointer was over.
		//
		//XDrawRectangle(wm->getDisplay(), menu.item_window, menu_gc, it.current()->item_x - 14, it.current()->item_y - 14, menu.width-1, it.current()->item_y + item_height - 15);
	
	}
	
	XDrawString(wm->getDisplay(), menu.title, menu_gc, 
   	   8, 14 , menu.menuTitle, strlen(menu.menuTitle));
}

// Redraws a single menu item on the menu.
void BaseMenu::redraw(BaseMenuItem *i)
{
	if(strcmp(i->name, "separator")==0) 
	{
		XDrawLine(wm->getDisplay(), menu.item_window, menu_gc, i->item_x - 14, i->item_y-5, menu.width-1, i->item_y-5);
		XDrawLine(wm->getDisplay(), menu.item_window, menu_gc, i->item_x - 14, i->item_y-6, menu.width-1, i->item_y-6);				
	} else {
	
		XDrawString(wm->getDisplay(), menu.item_window, menu_gc, 
		i->item_x, i->item_y + 2, i->name, strlen(i->name));
	}
	
	// Draws a bounding box around each menu item. This helped me figure out the algorithm to test which item the 
	// pointer was over.
	//
	//XDrawRectangle(wm->getDisplay(), menu.item_window, menu_gc, it.current()->item_x - 14, it.current()->item_y - 14, menu.width-1, it.current()->item_y + item_height - 15);

}

void BaseMenu::insert(char *n, BaseMenu *sub)
{
	BaseMenuItem *item = new BaseMenuItem();
		
	item->name = new char[strlen(n)+1];	
	strcpy(item->name, n);
	
	item->sub  = sub;

	// defaults
	item->exec = "";
	item->function = 0;
	item->item_x = item->item_y = 0;

	mi->insert(item);
}

void BaseMenu::insert(char *n, char *exec, int func)
{
	BaseMenuItem *item = new BaseMenuItem();
	
	char *buffer = new char[100];
        if (exec[0] == '~') 
	{
		sprintf(buffer, "%s%s", getenv("HOME"), exec+1);
		strcpy(exec, buffer);
	}
	delete [] buffer;
	
	item->exec = new char[strlen(exec)+1];
	strcpy(item->exec, exec);
	
	item->name = new char[strlen(n)+1];	
	strcpy(item->name, n);
	
	item->function = func;

	// defaults
	item->sub  = None;
	item->item_x = item->item_y = 0;

	mi->insert(item);
}


void BaseMenu::insert(char *n, char *exec, BaseMenu *sub, int func)
{
	BaseMenuItem *item = new BaseMenuItem();
	
	char *buffer = new char[100];
        if (exec[0] == '~') 
	{
		sprintf(buffer, "%s%s", getenv("HOME"), exec+1);
		strcpy(exec, buffer);
	}
	delete [] buffer;
	
	item->exec = new char[strlen(exec)+1];
	strcpy(item->exec, exec);
	
	item->name = new char[strlen(n)+1];	
	strcpy(item->name, n);
	
	item->sub  = sub;
	item->function = func;
	item->item_x = item->item_y = 0;
	
	mi->insert(item);
}

int BaseMenu::remove(int element)
{
   mi->remove(element);

  return mi->count();
}


void BaseMenu::remove_all()
{
}

void BaseMenu::execute(char *s)
{
	char command[strlen(s)+8];
	
	sprintf(command, "exec %s &", s);
	system(command);
}

void BaseMenu::show()
{	
	// Gets the coordinates of the mouse
	Client *c = NULL;
	c->get_mouse_position(&menu.x, &menu.y);
		
	// Check to make sure menu will be displayed on the screen.
 	if (menu.x + menu.width > wm->getXRes() - 1) {
 		menu.x = wm->getXRes() - menu.width - 1;
 	}
	
	if (menu.y + menu.height > wm->getYRes() - 1) {
		menu.y = wm->getYRes() - menu.height;
 	}

	// Move the menu to the position of the mouse pointer
	XMoveWindow(wm->getDisplay(), menu.window, menu.x, menu.y);
	// Show the menu windows
	XMapRaised(wm->getDisplay(), menu.window);
	XMapSubwindows(wm->getDisplay(),menu.window);
	
	isVisible=True;
}

void BaseMenu::show(int x, int y)
{	
	menu.x = x;
	menu.y = y;
	
	XMoveWindow(wm->getDisplay(), menu.window, x, y);

	// Show the menu windows
	XMapSubwindows(wm->getDisplay(),menu.window);
	XMapRaised(wm->getDisplay(), menu.window);

	isVisible=True;
}

// Doesn't work right yet!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void BaseMenu::showSub(BaseMenu *sub, int x, int y)
{
	int dummy_x=0, dummy_y=0;
	
	sub->menu.x = x;
	sub->menu.y = y;
	
	// Gets the coordinates of the mouse
	Client *c = NULL;
	c->get_mouse_position(&menu.x, &menu.y);
		
	// Check to make sure menu will be displayed on the screen.
 	if (dummy_x + menu.width > wm->getXRes() - 1) {
 		x = wm->getXRes() - menu.width - 1;
 	}
	
	if (dummy_y + menu.height > wm->getYRes() - 1) {
		y = wm->getYRes() - menu.height;
 	}
		
	XMoveWindow(wm->getDisplay(), sub->menu.window, x, y);

	// Show the menu windows
	XMapSubwindows(wm->getDisplay(), sub->menu.window);
	XMapRaised(wm->getDisplay(),  sub->menu.window);

	isVisible=True;
}

void BaseMenu::hide()
{
	// hide the menu windows
	XUnmapSubwindows(wm->getDisplay(), menu.window);
	XUnmapWindow(wm->getDisplay(), menu.window);

	isVisible=False;
}


BaseMenuItem *BaseMenu::findMenuItem(int x, int y)
{
	int item_xpos2, item_ypos2;
	
	if(mi->count()) 
	{
		LinkedListIterator<BaseMenuItem> it(mi);
		for(; it.current(); it++)
		{

			// Draws a bounding box around each menu item. This helped me figure out the algorithm to test which item the 
			// pointer was over.
			//
			//XDrawRectangle(wm->getDisplay(), menu.item_window, menu_gc, it.current()->item_x - 14, 
			//	it.current()->item_y - 14, menu.width-1, it.current()->item_y + item_height - 15);


			item_xpos2 = menu.width-1;
			item_ypos2 = it.current()->item_y + item_height - 15;
	  
			if ( (x >= it.current()->item_x - 14) && 
			     (x < item_xpos2) &&
			     (y >= it.current()->item_y - 14) &&  
			     (y < item_ypos2)
			   ) 
			    {
			    	
				    return it.current();
			    }
		}	
	}

	return NULL;
}

void BaseMenu::handle_button_press_event(XButtonEvent *e)
{
	switch (e->button) {
			case Button1:
			{
				if(e->window == menu.title) move();
				else if (curr) {
					handle_button1_press();
				}
				
				
			break;
			}

			case Button2:
			{
				if(curr)
				{
					handle_button2_press();
				}
	
			break;
			}
			
			case Button3:
			{
				if (curr) {
					handle_button3_press();
				}
			break;
			}
	} 

}

void BaseMenu::handle_button_release_event(XButtonEvent *e)
{
	switch (e->button) {
			case Button1:
			{
				if (curr) {
					handle_button1_release();
				
					// 99 means that it doesn't serve as a wm function
					if(curr->function == 99) 
					{
						execute(curr->exec);
						
					} else {
					
						switch(	curr->function )
						{
						
							// function 1 is wm shutdown
							case 1:
							{
								wm->quit_nicely();
							}
							break;
							
							case 2:
							{
								// do nothing this is a menu separator
							}
							break;
										
							default:
							{
								execute(curr->exec);
							}
							break;
						}										
				}

				
				}
				
			break;
			}

			case Button2:
			{
				if (curr) {
					handle_button2_release();
				}
			break;
			}
			
			case Button3:
			{
				if (curr) {
					handle_button3_release();
				}
				
				if(e->window == menu.title)
				{
					hide();
				}
			break;
			}
	} 
}

void BaseMenu::move()
{
	XEvent ev;
	
	int dx,dy;
	
	wm->grabPointer(wm->getRootWindow(), ButtonPressMask | ButtonReleaseMask | PointerMotionMask, wm->move_curs);
	
	XGrabServer(wm->getDisplay());
	
	for (;;) {
		XMaskEvent(wm->getDisplay(), ButtonPressMask | ButtonReleaseMask | PointerMotionMask, &ev);
		switch (ev.type) {
			case MotionNotify:
			{				
				dx = ev.xmotion.x_root - menu.x_move;
				dy = ev.xmotion.y_root - menu.y_move;
				menu.x = dx;
				menu.y = dy;
				XMoveWindow(wm->getDisplay(), menu.window, dx, dy);
				
				redraw();			
			}
			break;
			
			case ButtonRelease:
			{
				XSync(wm->getDisplay(), False);
				XUngrabServer(wm->getDisplay());
				wm->ungrabPointer();
				return;
			}

		}
	}

}

void BaseMenu::setMenuPos(int x, int y)
{
	menu.x = x; 
	menu.y = y;
	
	XMoveWindow(wm->getDisplay(), menu.window, x, y);
}

void BaseMenu::handle_enter_notify(XCrossingEvent *e)
{
	if(curr)
	{					
		XFillArc(wm->getDisplay(), menu.item_window, select_gc, curr->item_x-15, curr->item_y-15,
       			item_height - 1, item_height - 2, 90 * 64, 180 * 64);
    		
		XFillRectangle(wm->getDisplay(), menu.item_window, select_gc, curr->item_x - 15 +
       			((item_height) / 2), curr->item_y -15, item_width -
  				item_height - 2, item_height - 1);	 
		
		redraw(curr);
		
		//if(curr->sub) {
		//	if(! curr->sub->isVisible)
		//	{
		//		showSub(curr->sub,menu.x + menu.width + 1, menu.y + curr->item_y - 15);
		//	} 
		//} 
	}	
}

void BaseMenu::handle_leave_notify(XCrossingEvent *e)
{
	if(curr)
	{	
		XFillArc(wm->getDisplay(), menu.item_window, back_gc, curr->item_x-15, curr->item_y-15,
       			item_height - 1, item_height - 2, 90 * 64, 180 * 64);
    		
		XFillRectangle(wm->getDisplay(), menu.item_window, back_gc, curr->item_x - 15 +
       			((item_height) / 2), curr->item_y -15, item_width - item_height - 2, item_height - 1);	 

		redraw(curr);
		
		//if(curr->sub && curr->sub->isVisible)
		//{
			//LinkedListIterator<BaseMenuItem> it(mi);
			//for(; it.current(); it++)
			//{	  	
				//if (it.current()->sub && it.current()->sub->isVisible) it.current()->sub->hide();
			//} 
		//}
		curr = NULL;			
	}
}

void BaseMenu::handle_expose_event(XExposeEvent *e)
{
	if(e->count == 0) redraw();
}

void BaseMenu::handle_motion_notify_event(XMotionEvent *e)
{
	// for moving the menu.
	menu.x_move = e->x;
	menu.y_move = e->y;
	
	//----------------------
	XEvent temp;	
	
	if(e->window == menu.title) { /* DO NOTHING */ }
	else if(e->window == menu.item_window)
	{
		BaseMenuItem *i = findMenuItem(e->x, e->y);

		if (i && enterOnce)
		{
			curr = i;

			if(! strcmp(i->name, "separator")==0) 
			{	
				handle_enter_notify(&temp.xcrossing);				
			}

			enterOnce=False;
		} else {

			
			BaseMenuItem *i = findMenuItem(e->x, e->y);
			
			if (i != curr)
			{
			
				handle_leave_notify(&temp.xcrossing);
			
				curr =  i;
				
				enterOnce=True;
			} 
			
		}
	}	
}

void BaseMenu::handle_config_notify_event(XConfigureEvent *e)
{
}

void BaseMenu::reconfigure()
{	
	update_all();
	redraw();
}

void BaseMenu::createMenuImage()
{
	if((MenuImage.getWidth() != menu.width) || (MenuImage.getHeight() != menu.height))
	{
		MenuImage.setWidth(menu.width);
		MenuImage.setHeight(menu.height);
		MenuImage.setColorDepth(DefaultDepth(wm->getDisplay(), wm->screen));
		MenuImage.createImage();
	
		if(wm->theme->isMenuHorizontalGradient())
			MenuImage.renderHGradient(wm->theme->getMenuColorFrom(), wm->theme->getMenuColorTo());

		if(wm->theme->isMenuVerticalGradient())
			MenuImage.renderVGradient(wm->theme->getMenuColorFrom(), wm->theme->getMenuColorTo());
	
		if(wm->theme->isMenuSolid())
			MenuImage.setBackgroundColor(wm->theme->getMenuColor());

		if(wm->theme->isMenuBeveled())
			MenuImage.renderBevel();
		
		menu_pix = MenuImage.convertToPixmap(wm->getDisplay(), DefaultVisual(wm->getDisplay(), wm->screen), menu.item_window);

		XSetWindowBackgroundPixmap(wm->getDisplay(), menu.title, menu_pix);		
		XSetWindowBackgroundPixmap(wm->getDisplay(), menu.item_window, menu_pix);
	}
}

void BaseMenu::createMenuSelectImage()
{
	if((SelectImage.getWidth() != menu.width) || (SelectImage.getHeight() != item_height))
	{
		SelectImage.setWidth(menu.width);
		SelectImage.setHeight(item_height);
		SelectImage.setColorDepth(DefaultDepth(wm->getDisplay(), wm->screen));
		SelectImage.createImage();
	
		if(wm->theme->isMenuSelectHorizontalGradient())
			SelectImage.renderHGradient(wm->theme->getMenuSelectColorTo(), wm->theme->getMenuSelectColorFrom());

		if(wm->theme->isMenuSelectHorizontalGradient())
			SelectImage.renderVGradient(wm->theme->getMenuSelectColorTo(), wm->theme->getMenuSelectColorFrom());
	
		if(wm->theme->isMenuSelectSolid())
			SelectImage.setBackgroundColor(wm->theme->getMenuSelectColor());

		if(wm->theme->isMenuSelectBeveled())
			SelectImage.renderBevel();

		select_pix = SelectImage.convertToPixmap(wm->getDisplay(), DefaultVisual(wm->getDisplay(), wm->screen), menu.title);
	}
}

void BaseMenu::createMenuTitleImage()
{
	if((TitleImage.getWidth() != menu.width))
	{
		TitleImage.setWidth(menu.width);
		TitleImage.setHeight(menu.height);
		TitleImage.setColorDepth(DefaultDepth(wm->getDisplay(), wm->screen));
		TitleImage.createImage();
	
		if(wm->theme->isMenuTitleHorizontalGradient())
			TitleImage.renderHGradient(wm->theme->getMenuTitleColorTo(), wm->theme->getMenuTitleColorFrom());

		if(wm->theme->isMenuTitleVerticalGradient())
			TitleImage.renderHGradient(wm->theme->getMenuTitleColorTo(), wm->theme->getMenuTitleColorFrom());

		if(wm->theme->isMenuTitleSolid())
			TitleImage.setBackgroundColor(wm->theme->getMenuTitleColor());

		if(wm->theme->isMenuTitleBeveled())
			TitleImage.renderBevel();

		title_pix = TitleImage.convertToPixmap(wm->getDisplay(), DefaultVisual(wm->getDisplay(), wm->screen), menu.title);

		XSetWindowBackgroundPixmap(wm->getDisplay(), menu.title, title_pix);
	}
}

	

