// BaseMenu.hh
//
// Frank Hale 
// frankhale@yahoo.com
#ifndef _BASEBaseMenu_HH_
#define _BASEBaseMenu_HH_

class BaseMenu
{
private:	
	Image MenuImage;
	Image SelectImage;
	Image TitleImage;

	Pixmap menu_pix;
	Pixmap select_pix;
	Pixmap title_pix;

	bool isVisible;
	
	XSetWindowAttributes attrib;
	unsigned long create_mask;	

	GC menu_gc;
	GC select_gc;
	GC back_gc;
	XFontStruct *font;
	XCharStruct overall;
	int direction, ascent, descent;
	
	int counter;
	
	struct menu {
		char* menuTitle;
		Window title;
		Window window;
		Window item_window;
		int x,y;
		int x_move, y_move;
		unsigned int width,height;
		
		unsigned int total_item_height;
	} menu;
	
	unsigned int item_width,item_height;
	
	Cursor curs;
	
	// Used to know which item to paint.
	BaseMenuItem *curr;
	
	// Gotta fake a enter event and we only want this to happen once.
	bool enterOnce;	
	
public:	
	// Contains the menu items of this menu.
	LinkedList<BaseMenuItem> *mi;	
	
	BaseMenu();
	virtual ~BaseMenu();
		
	void setMenuTitle(char* t) { menu.menuTitle=t; }	
	void setMenuPos(int x, int y);
	void show();
	void show(int x, int y);
	void showSub(BaseMenu *sub, int x, int y);
	void hide();
	void update_all();
	void update_menu();
	void reconfigure();

	//LinkedList<BaseMenuItem>* getMenuItemList() const { return mi; }
	virtual void insert(char *n, BaseMenu *sub);
	virtual void insert(char *n, char *exec, int func);
	virtual void insert(char *n, char *exec, BaseMenu *sub, int func);
	int remove(int element);
	void remove_all();

	BaseMenuItem *findMenuItem(int x, int y);

	void handle_button_press_event(XButtonEvent *e);
	void handle_button_release_event(XButtonEvent *e);
	void handle_enter_notify(XCrossingEvent *e);
	void handle_leave_notify(XCrossingEvent *e);
	void handle_expose_event(XExposeEvent *e);
	void handle_motion_notify_event(XMotionEvent *e);
	void handle_config_notify_event(XConfigureEvent *e);

	// The menu item behavoir is defined with these
	// virtual functions in a derived class.
	virtual void handle_button1_press()   { /* Has no default behavior */ }
	virtual void handle_button2_press()   { /* Has no default behavior */ }
	virtual void handle_button3_press()   { /* Has no default behavior */ }
	virtual void handle_button1_release() { /* Has no default behavior */ }
	virtual void handle_button2_release() { /* Has no default behavior */ }
	virtual void handle_button3_release() { /* Has no default behavior */ }

	void execute(char *s);

	// Count relative to zero
	int getItemCount() const { return mi->count() - 1; }
	
	Window getMenuTitleWindow() const { return menu.title; }
	Window getMenuWindow() const { return menu.window; }
	Window getMenuItemWindow() const { return menu.item_window; }

private:

	void move();
	void redraw();
	void redraw(BaseMenuItem *m);	
	
	void createMenuImage();
	void createMenuSelectImage();
	void createMenuTitleImage();
};

#endif
