// Client.hh
//
// Purpose:
// 	Handles creation, life and death of a client window.
// 	Handles creation of window decorations, resizing, shading, etc...
// 
//
// Frank Hale
// frankhale@yahoo.com / frankhale@mailcity.com
// 15 Jan 1999
/////////////////////////////////////////////////////////////////////////
#ifndef _CLIENT_HH_

class Client {

private:
	
	struct button {
		Window window;
		int x,y;
		unsigned int width,height;
		bool pressed;
	};

private:	
	Image 		Title;
	Image 		Button;
	Image 		Focus;
	Image		Position;

	Window  	title;
	Window		window;
	Window		parent;
	Window		transient;
	Window		win_pos;   	// Window used to show x,y and size

	Pixmap 		frame_pix;	// Pixmap used for the titlebar
	Pixmap 		button_pix;	// Pixmap used for titlebar buttons
	Pixmap 		focus_pix;	// Pixmap used to show that the client has the focus
	Pixmap		position_pix;	// Pixmap for the client position window
	
	button 		shadeButton;
	button 		maximize;
	button 		minimize;
	button 		close;
	
	int		position_window_width;
	int		position_window_height;
	
	int 		default_button_width;
	int 		default_button_height;

	int 		default_close_button_x;
	int		default_close_button_y;
	
	int 		default_shade_button_x;
	int		default_shade_button_y;
	
	int 		default_minimize_button_x;
	int		default_minimize_button_y;	

	int 		default_maximize_button_x;
	int		default_maximize_button_y;

	int 		default_titlebar_height;
	
	int 		default_client_name_x;
	int		default_client_name_y;
	
	int		close_button_pressed;
	int 		shade_button_pressed;
	int		minimize_button_pressed;
	int 		maximize_button_pressed;
	
	char		*name;

	char		client_position[40];

	long 		state;

	Colormap	cmap;
	XSizeHints	*size;
	int		ignore_unmap;

	int		x;
	int		y;
	int		width;
	int		height;

	bool 		override_redirect;

	int		oumx;
	int		oumy;
	int		oumwidth;
	int		oumheight;

	bool 		button1;
	bool 		button2;
	bool 		button3;
	
	bool 		shaded;
	bool 		maxsized;

	// Titlebar turned on/off
	bool 		has_titlebar;
	
	// Client buttons turned on/off
	bool 		has_close_button;
	bool		has_minimize_button;
	bool		has_maximize_button;
	bool 		has_shade_button;
	
	bool 		has_focus;
	
	int 		proto;
	
	MwmHints *mwm_hint;
	
public: /* member functions */

	Client(); 
	~Client();

private:
	
	Window createChildWindow(Window parent, int x, int y, unsigned int width,
		unsigned int height,unsigned int borderwidth);
	
	Window createTopLevelWindow(int x, int y, unsigned int width,
		unsigned int height,unsigned int borderwidth);

	
	Client *find_client(Window w);
	void reparent(Client *c);
	void send_config(Client *c);
	void remove_client(Client *c);
	void resize(Client *c);
	void hide(Client *c);
	void move(Client *c);
	void unhide(Client *c);
	void change_gravity(Client *c, int multiplier);
	void shade(Client *c);
	void maximumresize(Client *c);
	
	void closeButtonPressed(Client *c);
	void shadeButtonPressed(Client *c);
	void minimizeButtonPressed(Client *c);
	void maximizeButtonPressed(Client *c);
	
	#ifdef SHAPE
		void set_shape(Client *c);
	#endif

	void drag(Client *c);
	void sweep(Client *c);
	void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2);
	void draw_outline(Client *c);
	void draw_shade_button(Client *c);
	int  get_wm_state(Client *c);
	void set_wm_state(Client *c, int state);
	void redraw(Client *c);
	void warpPointer(Window w, int x, int y);
	void installColorMap(Colormap cmap);
	void isIconic(Client *c);			// If this client is iconic the unmap it otherwise map it
	int isViewable(Client *c);			// Is this client visible?
	void mapClient(Client *c);			// Map Client to screen
	void getTransients(Client *c);			// Set this clients transient
	void getClientPosition(Client *c);
	
	// Client window decoration member functions
	void determineWindowDecorations(Client *c);	// Determines if this client needs to have a titlebar and what buttons to display.
	void do_gradient(Client *c);			// Creates the nice gradients for the window decorations
	void createTitleImage(Client* c);		// Creates the Title Image 
	void createButtonImage(Client* c);		// Creates the Button Image
	void createFocusImage(Client* c);		// Creates the Focus Image	
	void createPositionImage(Client* c);		// Creates the Position Image	

	void setButtonPositionDefaults(Client *c);
	
	void setCloseButtonPosition(Client *c, int x, int y);
	void setShadeButtonPosition(Client *c, int x, int y);
	void setMinimizeButtonPosition(Client *c, int x, int y);
	void setMaximizeButtonPosition(Client *c, int x, int y);
	void setClientNamePosition(Client *c, int x, int y);

	xy_pair getCloseButtonPosition(Client *c) const;
	xy_pair getShadeButtonPosition(Client *c) const;
	xy_pair getMinimizeButtonPosition(Client *c) const;
	xy_pair getMaximizeButtonPosition(Client *c) const;
	xy_pair getClientNamePosition(Client *c) const;
	
	void createParentWindow(Client *c);		// Creates the Parent Window
	void createTitlebarWindow(Client *c);		// Creates the Titlebar Window
	void createCloseButtonWindow(Client *c);	// Creates the Close Button Window
	void createMinimizeButtonWindow(Client *c);	// Creates the Minimize Button Window
	void createMaximizeButtonWindow(Client *c);	// Creates the Maximimize Button Window
	void createShadeButtonWindow(Client *c);	// Creates the Shade Button Window
	void createPositionWindow(Client *c);		// Creates the Position Window used to display the client's position

	void createRightHandleWindow(Client *c);
	void createLeftHandleWindow(Client *c);
	void createTopHandleWindow(Client *c);
	void createBottomHandleWindow(Client *c);

	int  theight(Client *c) const; 			// returns the titlebar height for a client
	
	int send_xmessage(Window w, Atom a, long x, long y);
	void send_wm_delete(Client *c);

	void reconfigure(Client *c);
		
public:
	void get_mouse_position(int *x, int *y);	
	void make_new_client(Window w);
	
	// Client Events
	void handle_unmap_event(XUnmapEvent *e);
	void handle_button_press_event(XButtonEvent *e);
	void handle_button_release_event(XButtonEvent *e);
	void handle_client_message(XClientMessageEvent *e);
	void handle_colormap_change(XColormapEvent *e);
	void handle_configure_request(XConfigureRequestEvent *e);
	void handle_enter_event(XCrossingEvent *e);
	void handle_expose_event(XExposeEvent *e);
	void handle_map_request(XMapRequestEvent *e);
	void handle_property_change(XPropertyEvent *e);
	void handle_reparent_event(XReparentEvent *e);
	void handle_destroy_notify(XDestroyWindowEvent *e);
	void handle_focus_in_event(XFocusChangeEvent *e);
	void handle_focus_out_event(XFocusChangeEvent *e);
	void handle_motion_event(XMotionEvent *e);
	
	#ifdef SHAPE
		void handle_shape_change(XShapeEvent *e);
	#endif

};

#endif 
