// Taken from blackbox version 0.8.6 by Brad Hughes
// http://blackbox.wiw.org
//
//
// The code has been seriously changed.
//
#include "sapphire.hh"

Image::Image()
{
	bc_default.r=255;
	bc_default.g=255;
	bc_default.b=255;
	
	width=height=depth=0;
	
	#ifdef DEBUG
		printf("Image default constructor called\n");
	#endif
}

Image::Image(unsigned int w, unsigned int h, int d, const Color &c) {
  width = w;
  height = h;
  depth = d;

  unsigned int wh = width * height;
  data = new unsigned long [wh];
  setBackgroundColor(c);
}

Image::~Image() {
  //if (colors_8bpp) delete [] colors_8bpp;
  //if (data) delete [] data;
}

void Image::createImage()
{
	unsigned int wh = width*height;
	data = new unsigned long [wh];
	setBackgroundColor(bc_default);	
}

XImage *Image::convertToXImage(Display *display, Visual *visual)
{ 
  colors_8bpp=0;

  char *d = new char[width * height * (depth / 8)];
  if (! d) return 0;

  XImage *image = XCreateImage(display, visual, depth, ZPixmap, 0, d, width,
			       height, 8, 0);
  
  if (image == NULL) return 0;

  unsigned char r, g, b;
  unsigned int wh = width * height;
  unsigned long *p = data;
    
  switch (depth) {
  
  case 32: 
  case 24: {
    
    unsigned char *im = (unsigned char *) image->data;

    while (--wh) {
      *(im++) = (unsigned char) ((*p) & 0xff);
      *(im++) = (unsigned char) (((*p) & 0xff00) >> 8);
      *(im++) = (unsigned char) (((*p++) & 0xff0000) >> 16);
    }

    *(im++) = (unsigned char)   ((*p) & 0xff);
    *(im++) = (unsigned char)   (((*p) & 0xff00) >> 8);
    *(im) =   (unsigned char)   (((*p) & 0xff0000) >> 16);

    break; }

  case 16: {
	unsigned short *im = (unsigned short *) image->data;

      // lets dither the image while blitting to the XImage
      unsigned int i, x, y, ofs, w2 = width * 2,
	aw = ((width + 2) * sizeof(unsigned short));
      unsigned short *tor, *tog, *tob, er, eg, eb, sr, sg, sb,
	*or = (unsigned short *) alloca(aw),
	*og = (unsigned short *) alloca(aw),
	*ob = (unsigned short *) alloca(aw),
	*nor = (unsigned short *) alloca(aw),
	*nog = (unsigned short *) alloca(aw),
	*nob = (unsigned short *) alloca(aw);
      
      if ((! or) || (! og) || (! ob) || (! nor) || (! nog) || (! nob)) {
	XDestroyImage(image);
	return 0;
      }

      tor = or; tog = og; tob = ob;
      
      for (i = 0; i < width; i++) {
	*(tor++) = ((*(data + i)) & 0xff0000) >> 16;
	*(tog++) = ((*(data + i)) & 0xff00) >> 8;
	*(tob++) = ((*(data + i)) & 0xff);
      }

      ofs = 0;
      for (y = 0; y < height; y++) {
	if (y < (height - 1)) {
	  tor = nor; tog = nog; tob = nob;
	  
	  for (i = ofs + width; i < ofs + w2; i++) {
	    *(tor++) = ((*(data + i)) & 0xff0000) >> 16;
	    *(tog++) = ((*(data + i)) & 0xff00) >> 8;
	    *(tob++) = ((*(data + i)) & 0xff);
	  }
	  
	  *tor = ((*(data + i - 1)) & 0xff0000) >> 16;
	  *tog = ((*(data + i - 1)) & 0xff00) >> 8;
	  *tob = ((*(data + i - 1)) & 0xff);
	}

	for (x = 0; x < width; x++) {
	  if (*(or + x) > 255) *(or + x) = 255;
	  if (*(og + x) > 255) *(og + x) = 255;
	  if (*(ob + x) > 255) *(ob + x) = 255;
	  
	  sr = *(or + x);
	  sg = *(og + x);
	  sb = *(ob + x);

	  sr = ((sr << 8) & image->red_mask);
	  sg = ((sg << 3) & image->green_mask);
	  sb = ((sb >> 3) & image->blue_mask);
	  
	  *(im++) = sr|sg|sb;
	  
	  er = *(or + x) - (sr >> 8);
	  eg = *(og + x) - (sg >> 3);
	  eb = *(ob + x) - (sb << 3);
	  
	  *(or + x + 1) += ((er >> 3) + (er >> 2));
	  *(og + x + 1) += ((eg >> 3) + (eg >> 2));
	  *(ob + x + 1) += ((eb >> 3) + (eb >> 2));
	  
	  *(nor + x) += ((er >> 3) + (er >> 2));
	  *(nog + x) += ((eg >> 1) + (eg >> 2));
	  *(nob + x) += ((eb >> 1) + (eg >> 2));
	  
	  *(nor + x + 1) += (er >> 2);
	  *(nog + x + 1) += (eg >> 2);
	  *(nob + x + 1) += (eb >> 2);
	}

	ofs += width;
	
	tor = or; tog = og; tob = ob;
	or = nor; og = nog; ob = nob;
	nor = tor; nog = tog; nob = tob;
	
      }

    break; }
    
  case 15: 
  case 8: {
     
    colors_8bpp = new XColor[125];
    int i = 0;
    for (r = 0; r < 5; r++)
      for ( g = 0; g < 5; g++)
	for ( b = 0; b < 5; b++) {
	  colors_8bpp[i].red = (r * 0xffff) / 4;
	  colors_8bpp[i].green = (g * 0xffff) / 4;
	  colors_8bpp[i].blue = (b * 0xffff) / 4;
	  colors_8bpp[i++].flags = DoRed|DoGreen|DoBlue;
	}
    
    XGrabServer(display);
    
    Colormap colormap = DefaultColormap(display, DefaultScreen(display));
    for (i = 0; i < 125; i++)
      if (! XAllocColor(display, colormap, &colors_8bpp[i])) {
	fprintf(stderr, "couldn't alloc color %i %i %i\n", colors_8bpp[i].red,
		colors_8bpp[i].green, colors_8bpp[i].blue);		
	colors_8bpp[i].flags = 0;
      } else
	colors_8bpp[i].flags = DoRed|DoGreen|DoBlue;

    XUngrabServer(display);
  
    int rr, gg, bb;
    unsigned char *im = (unsigned char *) image->data;

    while (--wh) {
      r = (unsigned char) (((*p) & 0xff0000) >> 16);
      g = (unsigned char) (((*p) & 0xff00) >> 8);
      b = (unsigned char) ((*(p++)) & 0xff);

      rr = (r * 5) / 0xff;
      gg = (g * 5) / 0xff;
      bb = (b * 5) / 0xff;

      if (rr < 0) rr = 0;
      else if (rr > 4) rr = 4;
      if (gg < 0) gg = 0;
      else if (gg > 4) gg = 4;
      if (bb < 0) bb = 0;
      else if (bb > 4) bb = 4;
      *(im++) = colors_8bpp[(rr * 25) + (gg * 5) + bb].pixel;
    }

    r = (unsigned char) (((*p) & 0xff0000) >> 16);
    g = (unsigned char) (((*p) & 0xff00) >> 8);
    b = (unsigned char) ((*(p++)) & 0xff);
    
    rr = (r * 5) / 0xff;
    gg = (g * 5) / 0xff;
    bb = (b * 5) / 0xff;
    
    if (rr < 0) rr = 0;
    else if (rr > 4) rr = 4;
    if (gg < 0) gg = 0;
    else if (gg > 4) gg = 4;
    if (bb < 0) bb = 0;
    else if (bb > 4) bb = 4;
    *(im) = colors_8bpp[(rr * 25) + (gg * 5) + bb].pixel;
    

    break; }

  }

  return image;
}


Pixmap Image::convertToPixmap(Display *display, Visual *visual, Window window)
{
  Pixmap pixmap = XCreatePixmap(display, window, width, height,
			 (unsigned) depth);
  
  if (pixmap == None) return None;

  XImage *img = convertToXImage(display, visual);

  if (! img) {
    XFreePixmap(display, pixmap);
    return None;
  }

  XPutImage(display, pixmap, DefaultGC(display, DefaultScreen(display)),
	    img, 0, 0, 0, 0, width, height);

  XDestroyImage(img);

  if(data) delete [] data;
  if (colors_8bpp) delete [] colors_8bpp;

  return pixmap;
}


void Image::setBackgroundColor(const Color &c) {
  bg_color = c;

  unsigned int wh = width * height;
  unsigned long *p = data, pixel = ((c.r << 16)|(c.g << 8)|(c.b));
  while (--wh) *(p++) = pixel;
  *p = pixel;
}

  
void Image::setBackgroundColor(unsigned char r, unsigned char g,
				unsigned char b)
{
  bg_color.r = r;
  bg_color.g = g;
  bg_color.b = b;

  unsigned int wh = width * height; 
  unsigned long *p = data, pixel = (r << 16) | (g << 8) | (b);

  while (--wh) *(p++) = pixel;
  *p = pixel;
}


void Image::renderBevel(void) {
  if (height > 4) {
  unsigned long pix0, pix1;
  unsigned char r, g, b;

  r = bg_color.r * 5 / 4;
  if (r < bg_color.r) r = 0xff;
  g = bg_color.g * 5 / 4;
  if (g < bg_color.g) g = 0xff;
  b = bg_color.b * 5 / 4;
  if (b < bg_color.b) b = 0xff;
  
  pix0 = ((r << 16)|(g << 8)|b);
  
  r = bg_color.r * 3 / 4;
  if (r > bg_color.r) r = 0x0;
  g = bg_color.g * 3 / 4;
  if (g > bg_color.g) g = 0x0;
  b = bg_color.b * 3 / 4;
  if (b > bg_color.b) b = 0x0;

  pix1 = ((r << 16)|(g << 8)|b);

  // lets do the easy parts... top and bottom lines
  unsigned int w = width - 2, h = height - 1, wh = width * (height - 3);

  unsigned long *p = data + width + 1;

  while (--w) {
    *p = pix0;
    *((p++) + wh) = pix1;
  }

  p = data + width;
  while (--h) {
    *(++p) = pix0;
    p += (width - 3);
    *(p++) = pix1;
    p++;
  }
}
}

void Image::invertImage(void) {
  unsigned int wh = width * height;
  unsigned long *p, *pp, *new_data = new unsigned long[wh];
  pp = new_data;
  p = data + wh;

  while (--wh) *(pp++) = *(p--);
  *pp = *p;
  
  delete [] data;
  data = new_data;
}

void Image::renderHGradient(const Color &from, const Color &to) {
  float fr, fg, fb, tr, tg, tb, dr, dg, db, dx, xr, xg, xb;
  
  fr = (float) from.r;
  fg = (float) from.g;
  fb = (float) from.b;

  tr = (float) to.r;
  tg = (float) to.g;
  tb = (float) to.b;

  dr = tr - fr;
  dg = tg - fg;
  db = tb - fb;

  unsigned long *p = data, *d = new unsigned long[width], *dd = d;
  unsigned char r = from.r, g = from.g, b = from.b;

  // this renders one line of the hgradient to a buffer...

  unsigned int ii;
  float w = (float) width;
  for (ii = 0; ii < width; ++ii) {
    dx = ii / w;

    xr = (dr * dx) + fr;
    xg = (dg * dx) + fg;
    xb = (db * dx) + fb;

    r = (unsigned char) (xr);
    g = (unsigned char) (xg);
    b = (unsigned char) (xb);
    *(dd++) = (unsigned long) ((r << 16) | (g << 8) | (b));
  }

  // and this copies the buffer to the image...
  unsigned int wh = width * height;

  ii = 0;
  while (--wh) {
    *(p++) = *(d + (ii++));
    if (ii == width) ii = 0;
  }

  *p = *(d + ii);
  delete [] d;
}


void Image::renderVGradient(const Color &from, const Color &to) {
  float fr, fg, fb, tr, tg, tb, dr, dg, db, dy, yr, yg, yb;
  
  fr = (float) from.r;
  fg = (float) from.g;
  fb = (float) from.b;

  tr = (float) to.r;
  tg = (float) to.g;
  tb = (float) to.b;

  dr = tr - fr;
  dg = tg - fg;
  db = tb - fb;

  unsigned long *p = data;
  unsigned char r = from.r, g = from.g, b = from.b;
  unsigned int ii;
  float h = (float) height;

  for (ii = 0; ii < height; ++ii) {
    dy = ii / h;

    yr = (dr * dy) + fr;
    yg = (dg * dy) + fg;
    yb = (db * dy) + fb;
      
    r = (unsigned char) (yr);
    g = (unsigned char) (yg);
    b = (unsigned char) (yb);

    unsigned int w = width;
    while (--w)
      *(p++) = (unsigned long) ((r << 16) | (g << 8) | (b));


    *(p++) = (unsigned long) ((r << 16) | (g << 8) | (b));
  }
}





















