// sapphire.hh
// 
// Purpose:
//
// Declares all the globals and things which sapphire needs to know about.
//
// Frank Hale 
// frankhale@yahoo.com
//
// Updated: 20 Jan 2000
///////////////////////////////////////////////////////////////////////////////
#ifndef _SAPPHIRE_HH_
#define _SAPPHIRE_HH_

static char* windowmanager_author = "Frank Hale";
static char* windowmanager_author_email = "frankhale@yahoo.com";
static char* windowmanager_name = "Sapphire";
static char* windowmanager_version = "Sapphire 0.13.3";
static char* windowmanager_usage = "[-display <display>] [-fn <font>]\n[-fg foreground] [-bg background] [-bd border] [-version]\n";
static char* windowmanager_exit_message = "\nThank you for using Sapphire.\n\nPlease Send all bug reports, suggestions, ideas, patches to frankhale@yahoo.com\n";

//#define DEBUG
#define GNOME

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <stdarg.h>

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>
#include <X11/Xresource.h>
//#include <X11/xpm.h>

#ifdef SHAPE
#include <X11/extensions/shape.h>
#endif

static char* HOME = getenv("HOME");

/* default settings */
#define DEFAULT_FONT			"-*-lucida-medium-r-normal-*-15-*-*-*-*-*-*-*"
#define MENU_FONT			"-*-lucida-medium-r-normal-*-15-*-*-*-*-*-*-*"
#define DEFAULT_FOREGROUND_COLOR 	"black"
#define DEFAULT_BACKGROUND_COLOR	"lightgray"
#define DEFAULT_BORDER_COLOR		"black"
#define DEFAULT_BORDER_SIZE		1
#define DEFAULT_SPACE			3
#define DEFAULT_MARGIN			6
#define DEFAULT_MINIMUM_SIZE		10

// GNOME hints
#define WIN_STATE_STICKY          (1<<0) /*everyone knows sticky*/
#define WIN_STATE_MINIMIZED       (1<<1) /*Reserved - definition is unclear*/
#define WIN_STATE_MAXIMIZED_VERT  (1<<2) /*window in maximized V state*/
#define WIN_STATE_MAXIMIZED_HORIZ (1<<3) /*window in maximized H state*/
#define WIN_STATE_HIDDEN          (1<<4) /*not on taskbar but window visible*/
#define WIN_STATE_SHADED          (1<<5) /*shaded (MacOS / Afterstep style)*/
#define WIN_STATE_HID_WORKSPACE   (1<<6) /*not on current desktop*/
#define WIN_STATE_HID_TRANSIENT   (1<<7) /*owner of transient is hidden*/
#define WIN_STATE_FIXED_POSITION  (1<<8) /*window is fixed in position even*/
#define WIN_STATE_ARRANGE_IGNORE  (1<<9) /*ignore for auto arranging*/

#define WIN_HINTS_SKIP_FOCUS      (1<<0) /*"alt-tab" skips this win*/
#define WIN_HINTS_SKIP_WINLIST    (1<<1) /*do not show in window list*/
#define WIN_HINTS_SKIP_TASKBAR    (1<<2) /*do not show on taskbar*/
#define WIN_HINTS_GROUP_TRANSIENT (1<<3) /*Reserved - definition is unclear*/
#define WIN_HINTS_FOCUS_ON_CLICK  (1<<4) /*app only accepts focus if clicked*/

#define WIN_LAYER_DESKTOP                0
#define WIN_LAYER_BELOW                  2
#define WIN_LAYER_NORMAL                 4
#define WIN_LAYER_ONTOP                  6
#define WIN_LAYER_DOCK                   8
#define WIN_LAYER_ABOVE_DOCK             10
#define WIN_LAYER_MENU                   12

// MOTIF hints
#define MwmHintsFunctions     (1l << 0)
#define MwmHintsDecorations   (1l << 1)

#define MwmFuncAll            (1l << 0)
#define MwmFuncResize         (1l << 1)
#define MwmFuncMove           (1l << 2)
#define MwmFuncIconify        (1l << 3)
#define MwmFuncMaximize       (1l << 4)
#define MwmFuncClose          (1l << 5)

#define MwmDecorAll           (1l << 0)
#define MwmDecorBorder        (1l << 1)
#define MwmDecorHandle        (1l << 2)
#define MwmDecorTitle         (1l << 3)
#define MwmDecorMenu          (1l << 4)
#define MwmDecorIconify       (1l << 5)
#define MwmDecorMaximize      (1l << 6)

#define PropMwmHintsElements  3

// this structure only contains 3 elements... the Motif 2.0 structure contains
// 5... we only need the first 3... so that is all we will define
typedef struct MwmHints {
  unsigned long flags, functions, decorations;
} MwmHints;

enum {
	Pdelete = 1,
	Ptakefocus = 2
};

// Used by the Client class
struct xy_pair {
	int x, y;
};
	
extern int handle_xerror(Display *dpy, XErrorEvent *e);
extern void err(char *fmt, ...);
extern void show_event(XEvent e);
extern void signalhandler(int i);

class Image;
class WindowManager;
class Client;
class BaseMenuItem;
class BaseMenu;
class RootMenu;
class Toolbar;
class Theme;

#include "linkedlist.hh"
#include "graphics.hh"
#include "theme.hh"
#include "basemenuitem.hh"
#include "basemenu.hh"
#include "client.hh"
#include "windowmanager.hh"
#include "toolbar.hh"
#include "rootmenu.hh"

#endif
