// File:
// 
// Theme.cc
//
// Purpose:
//
// This file declares the class Theme. The Theme class is resposible for all theme configuration.
// this class will hold all properties which define a theme for this window manager.
//
// Future:
//
// This class will make it extremely easy to interface with configuration parsing code either developed
// from Flex/Bison or from other code used to parse configuration files. This code will only have to 
// interface with this file. The Client class which represents a client window in the window manager 
// uses the Theme class directly and never needs to know about configuration files or what have you.
// 
// Also since the client class uses this class directly any changes to the theme will be updated in real 
// time. The client reconfigures itself and uses the information contained in a theme object. If that data
// changes which would be the case if a graphical configuration tool were created to configure Sapphire then
// the windows and toolbar would automatically show these changes.
//
#ifndef _THEME_HH_
#define _THEME_HH_

class Theme
{
private:
	
	// These represet a color pair which is color_from and color_to
	// used for the gradients.
	Color client[2];
	Color titlebarButtons[2];
	Color menu[2];
	Color menu_title[2];
	Color menu_select[2];
	Color toolbar[2];
	Color root[2];
	Color client_focus[2];
	Color client_position[2];
	
	bool toolbarTop;
	bool toolbarVisible;
	
	bool titlebarHorizontalGradient;
	bool titlebarVerticalGradient;
	bool titlebarSolid;
	bool titlebarBeveled;

	bool buttonHorizontalGradient;
	bool buttonVerticalGradient;
	bool buttonSolid;
	bool buttonBeveled;

	bool toolbarHorizontalGradient;
	bool toolbarVerticalGradient;
	bool toolbarSolid;
	bool toolbarBeveled;
	
	bool clockHorizontalGradient;
	bool clockVerticalGradient;
	bool clockSolid;
	bool clockBeveled;
	
	bool menuHorizontalGradient;
	bool menuVerticalGradient;
	bool menuSolid;
	bool menuBeveled;
	
	bool menuTitleHorizontalGradient;
	bool menuTitleVerticalGradient;
	bool menuTitleSolid;
	bool menuTitleBeveled;
	
	bool menuSelectHorizontalGradient;
	bool menuSelectVerticalGradient;
	bool menuSelectSolid;
	bool menuSelectBeveled;
	
	bool positionHorizontalGradient;
	bool positionVerticalGradient;
	bool positionSolid;
	bool positionBeveled;	
	
public:
	Theme();
	~Theme();

	bool isPositionHorizontalGradient() const { return positionHorizontalGradient; }
	bool isPositionVerticalGradient() const { return positionVerticalGradient; }
	bool isPositionSolid() const { return positionSolid; }
	bool isPositionBeveled() const { return positionBeveled; }

	bool isTitlebarHorizontalGradient() const { return titlebarHorizontalGradient; }
	bool isTitlebarVerticalGradient() const { return titlebarVerticalGradient; }
	bool isTitlebarSolid() const { return titlebarSolid; }
	bool isTitlebarBeveled() const { return titlebarBeveled; }
	
	bool isButtonHorizontalGradient() const { return buttonHorizontalGradient; }
	bool isButtonVerticalGradient() const { return buttonVerticalGradient; }
	bool isButtonSolid() const { return buttonSolid; }
	bool isButtonBeveled() const { return buttonBeveled; }

	bool isToolbarHorizontalGradient() const { return toolbarHorizontalGradient; }
	bool isToolbarVerticalGradient() const { return toolbarVerticalGradient; }
	bool isToolbarSolid() const { return toolbarSolid; }
	bool isToolbarBeveled() const { return toolbarBeveled; }

	bool isClockHorizontalGradient() const { return clockHorizontalGradient; }
	bool isClockVerticalGradient() const { return clockVerticalGradient; }
	bool isClockSolid() const { return clockSolid; }
	bool isClockBeveled() const { return clockBeveled; }
	
	bool isMenuHorizontalGradient() const { return menuHorizontalGradient; }
	bool isMenuVerticalGradient() const { return menuVerticalGradient; }
	bool isMenuSolid() const { return menuSolid; }
	bool isMenuBeveled() const { return menuBeveled; }

	bool isMenuSelectHorizontalGradient() const { return menuSelectHorizontalGradient; }
	bool isMenuSelectVerticalGradient() const { return menuSelectVerticalGradient; }
	bool isMenuSelectSolid() const { return menuSelectSolid; }
	bool isMenuSelectBeveled() const { return menuSelectBeveled; }
	
	bool isMenuTitleHorizontalGradient() const { return menuTitleHorizontalGradient; }
	bool isMenuTitleVerticalGradient() const { return menuTitleVerticalGradient; }
	bool isMenuTitleSolid() const { return menuTitleSolid; }
	bool isMenuTitleBeveled() const { return menuTitleBeveled; }

	Color getPositionColorFrom() const { return client_position[0]; }
	Color getPositionColorTo() const { return client_position[1]; }
	Color getPositionColor() const { return client_position[0]; }
		
	Color getClientColorFrom() const { return client[0]; }
	Color getClientColorTo() const { return client[1]; }
	Color getClientColor() const { return client[0]; }
	
	Color getClientFocusColorFrom() const { return client_focus[0]; }
	Color getClientFocusColorTo() const { return client_focus[1]; }	
	Color getClientFocusColor() const { return client_focus[0]; }
	
	Color getTitleBarButtonColorFrom() const { return titlebarButtons[0]; }
	Color getTitleBarButtonColorTo() const { return titlebarButtons[1]; }
	Color getTitleBarButtonColor() const { return titlebarButtons[0]; }
	
	Color getMenuColorFrom() const { return menu[0]; } 
	Color getMenuColorTo() const { return menu[1]; }
	Color getMenuColor() const { return menu[0]; } 

	Color getMenuTitleColorFrom() const { return menu_title[0]; } 
	Color getMenuTitleColorTo() const { return menu_title[1]; }
	Color getMenuTitleColor() const { return menu_title[0]; } 

	Color getMenuSelectColorFrom() const { return menu_select[0]; }
	Color getMenuSelectColorTo() const { return menu_select[1]; }
	Color getMenuSelectColor() const { return menu_select[0]; }

	Color getToolbarColorFrom() const { return toolbar[0]; }
	Color getToolbarColorTo() const {return toolbar[1]; }
	Color getToolbarColor() const { return toolbar[0]; }
	
	Color getRootColorFrom() const { return root[0]; }
	Color getRootColorTo() const { return root[1]; }
	Color getRootColor() const { return root[0]; }

	void setPositionColorFrom(Color c);
	void setPositionColorTo(Color c);
	void setPositionColor(Color c);
		
	void setClientColorFrom(Color c);
	void setClientColorTo(Color c);
	void setClientColor(Color c);

	void setClientFocusColorFrom(Color c);
	void setClientFocusColorTo(Color c);
	void setClientFocusColor(Color c);
	
	void setTitlebarButtonColorFrom(Color c);
	void setTitlebarButtonColorTo(Color c);
	void setTitlebarButtonColor(Color c);
	
	void setMenuColorFrom(Color c);
	void setMenuColorTo(Color c);
	void setMenuColor(Color c);

	void setMenuTitleColorFrom(Color c);
	void setMenuTitleColorTo(Color c);
	void setMenuTitleColor(Color c);

	void setMenuSelectColorFrom(Color c);
	void setMenuSelectColorTo(Color c);
	void setMenuSelectColor(Color c);

	void setToolbarColorFrom(Color c);
	void setToolbarColorTo(Color c);
	void setToolbarColor(Color c);
	
	void setRootColor(Color c);
	void setRootColorFrom(Color c);	
	void setRootColorTo(Color c);
	
	bool getToolbarVisible() const { return toolbarVisible; }
	void setToolbarVisible(bool vis) { toolbarVisible = vis; }
	
	bool getToolbarLocationTop() const {return toolbarTop; }
	void setToolbarLocationTop(bool s);

	void setPositionSolidColor(bool s) { positionSolid=s; }
	void setPositionHorizontalGradient(bool s) { positionHorizontalGradient=s; }
	void setPositionVerticalGradient(bool s) { positionVerticalGradient=s; }
	void setPositionBeveled(bool s) { positionBeveled=s; }
	
	void setTitlebarSolidColor(bool s) { titlebarSolid=s; }
	void setTitlebarHorizontalGradient(bool s) { titlebarHorizontalGradient=s; }
	void setTitlebarVerticalGradient(bool s) { titlebarVerticalGradient=s; }
	void setTitlebarBeveled(bool s) { titlebarBeveled=s; }
	
	void setButtonSolidColor(bool s) { buttonSolid=s; }
	void setButtonHorizontalGradient(bool s) { buttonHorizontalGradient=s; }
	void setButtonVerticalGradient(bool s) { buttonVerticalGradient=s; }
	void setButtonBeveled(bool s) { buttonBeveled=s; }

	void setToolbarSolidColor(bool s) { toolbarSolid=s; }
	void setToolbarHorizontalGradient(bool s) { toolbarHorizontalGradient=s; }
	void setToolbarVerticalGradient(bool s) { toolbarVerticalGradient=s; }
	void setToolbarBeveled(bool s) { toolbarBeveled=s; }

	void setMenuTitleSolidColor(bool s) { menuTitleSolid=s; }
	void setMenuTitleHorizontalGradient(bool s) { menuTitleHorizontalGradient=s; }
	void setMenuTitleVerticalGradient(bool s) { menuTitleVerticalGradient=s; }
	void setMenuTitleBeveled(bool s) { menuTitleBeveled=s; }
		
	void setMenuSolidColor(bool s) { menuSolid=s; }
	void setMenuHorizontalGradient(bool s) { menuHorizontalGradient=s; }
	void setMenuVerticalGradient(bool s) { menuVerticalGradient=s; }
	void setMenuBeveled(bool s) { menuBeveled=s; }

	void setMenuSelectSolidColor(bool s) { menuSelectSolid=s; }
	void setMenuSelectHorizontalGradient(bool s) { menuSelectHorizontalGradient=s; }
	void setMenuSelectVerticalGradient(bool s) { menuSelectVerticalGradient=s; }
	void setMenuSelectBeveled(bool s) { menuSelectBeveled=s; }

	void setClockSolidColor(bool s) { clockSolid=s; }
	void setClockHorizontalGradient(bool s) { clockHorizontalGradient=s; }
	void setClockVerticalGradient(bool s) { clockVerticalGradient=s; }
	void setClockBeveled(bool s) { clockBeveled=s; }
};

#endif
