// toolbar.cc
//
// Purpose:
//
// Provides a rudimentary toolbar. It displays 3 buttons and a clock.
// There is a close button and 2 buttons to cycle through the window stack.
//
// 
// 
//
//
// Frank Hale 
// frankhale@yahoo.com
#include "sapphire.hh"

Toolbar::Toolbar()
{
	font_width=0;

	month_abbreviation[0] = "Jan";
	month_abbreviation[1] = "Feb";
	month_abbreviation[2] =	"Mar";
	month_abbreviation[3] =	"Apr";
	month_abbreviation[4] =	 "May";
	month_abbreviation[5] =	 "Jun";
	month_abbreviation[6] =	 "Jul";
	month_abbreviation[7] =	 "Aug";
	month_abbreviation[8] =	 "Sep";
	month_abbreviation[9] =	 "Oct";
	month_abbreviation[10] = "Nov";
	month_abbreviation[11] = "Dec";

  	XSetWindowAttributes attrib;
  	unsigned long create_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
    		CWOverrideRedirect | CWEventMask;
  	attrib.background_pixmap = None;
  	attrib.background_pixel = wm->background_color.pixel;
  	attrib.border_pixel = wm->border_color.pixel;
  	attrib.override_redirect = True;
  	attrib.event_mask = StructureNotifyMask | ButtonPressMask | ButtonReleaseMask | ExposureMask;

	// check to see if toolbar is to be at top or bottom of the screen
  	// then place toolbar in correct location
	if (wm->theme->getToolbarLocationTop())
	{
  		frame.width = wm->getXRes() - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = 0;
  	} else {
  		frame.width = wm->getXRes() - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = wm->getYRes() - frame.height;
	}
  
  	frame.window =
    		XCreateWindow(wm->getDisplay(), wm->getRootWindow(), frame.x, frame.y,
		  frame.width, frame.height, 1, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);

	// Clock window --------------------------------------------------------------------
  	clock.x = wm->getXRes() - 160;
  	clock.y =  4;
  	clock.width = 130; // its hard coded to the size of the default font. 
  	clock.height = 15;

  	clock.window =  XCreateWindow(wm->getDisplay(), frame.window, clock.x, clock.y,
		  clock.width, clock.height, 1, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);
 
	// End Clock window -----------------------------------------------------------------

  	
	// Close Button ---------------------------------------------------------------------
  	close.x = 10;
	close.y = 4; 
	close.width = 15;
  	close.height = 15;

  	close.window =
    		 XCreateWindow(wm->getDisplay(), frame.window, close.x, close.y,
		  close.width, close.height, 1, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);

	// End Close Button -----------------------------------------------------------------

	// cycleUp Button ---------------------------------------------------------------------
  	cycleUp.x = 36;
	cycleUp.y = 4;
	cycleUp.width = 15;
  	cycleUp.height = 15;

  	cycleUp.window =
    		XCreateWindow(wm->getDisplay(), frame.window, cycleUp.x, cycleUp.y,
		  cycleUp.width, cycleUp.height, 1, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);

	// End cycleUp Button -----------------------------------------------------------------

	// cycleDown Button ---------------------------------------------------------------------
  	cycleDown.x = 54;
	cycleDown.y = 4;
	cycleDown.width = 15;
  	cycleDown.height = 15;

  	cycleDown.window =
    		XCreateWindow(wm->getDisplay(), frame.window, cycleDown.x, cycleDown.y,
		  cycleDown.width, cycleDown.height, 1, DefaultDepth(wm->getDisplay(), wm->screen),
		  InputOutput, DefaultVisual(wm->getDisplay(), wm->screen), create_mask, &attrib);

	// End cycleDown Button -----------------------------------------------------------------	
	
	reconfigure();
	
	if( wm->theme->getToolbarVisible())
	{
		XMapSubwindows(wm->getDisplay(), frame.window);
		XMapWindow(wm->getDisplay(), frame.window);
	}
}

Toolbar::~Toolbar()
{
	XFreePixmap(wm->getDisplay(), clock_pix);
	XFreePixmap(wm->getDisplay(), frame_pix);
	XFreePixmap(wm->getDisplay(), button_pix);
}

void Toolbar::createToolBarImage()
{
	if((ToolBarImage.getWidth() != frame.width) || (ToolBarImage.getHeight() != frame.height))
	{
		ToolBarImage.setWidth(frame.width);
		ToolBarImage.setHeight(frame.height);
		ToolBarImage.setColorDepth(DefaultDepth(wm->getDisplay(), wm->screen));
		ToolBarImage.createImage();
		
		if(wm->theme->isToolbarHorizontalGradient())
			ToolBarImage.renderHGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());		
		 
		if(wm->theme->isToolbarVerticalGradient())
			ToolBarImage.renderVGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());		
	
		if(wm->theme->isToolbarSolid())
			ToolBarImage.setBackgroundColor(wm->theme->getToolbarColor());		

		if(wm->theme->isToolbarBeveled())
			ToolBarImage.renderBevel();
	
		frame_pix = ToolBarImage.convertToPixmap(wm->getDisplay(), DefaultVisual(wm->getDisplay(), wm->screen), frame.window);			
	}
}

void Toolbar::createButtonImage()
{
	if((ButtonImage.getWidth() != close.width) || (ButtonImage.getHeight() != close.height))
	{
		ButtonImage.setWidth(close.width);
		ButtonImage.setHeight(close.height);
		ButtonImage.setColorDepth(DefaultDepth(wm->getDisplay(), wm->screen));
		ButtonImage.createImage();
		
		if(wm->theme->isButtonHorizontalGradient())
			ButtonImage.renderHGradient(wm->theme->getTitleBarButtonColorFrom(),wm->theme->getTitleBarButtonColorTo());

		if(wm->theme->isButtonVerticalGradient())
			ButtonImage.renderVGradient(wm->theme->getTitleBarButtonColorFrom(),wm->theme->getTitleBarButtonColorTo());	

		if(wm->theme->isButtonSolid())
			ButtonImage.setBackgroundColor(wm->theme->getTitleBarButtonColor());

		if(wm->theme->isButtonBeveled())
			ButtonImage.renderBevel();
	
        	button_pix = ButtonImage.convertToPixmap(wm->getDisplay(), DefaultVisual(wm->getDisplay(), wm->screen), close.window);
	}	
}

void Toolbar::createClockImage()
{
	if((ClockImage.getWidth() != clock.width) || (ClockImage.getHeight() != clock.height))
	{
		ClockImage.setWidth(clock.width);
		ClockImage.setHeight(clock.height);
		ClockImage.setColorDepth(DefaultDepth(wm->getDisplay(), wm->screen));
		ClockImage.createImage();
		
		if(wm->theme->isClockHorizontalGradient())
			ClockImage.renderHGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());
		
		if(wm->theme->isClockVerticalGradient())
			ClockImage.renderVGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());
		
		if(wm->theme->isClockSolid())
			ClockImage.setBackgroundColor(wm->theme->getToolbarColor());
			
		if(wm->theme->isClockBeveled())	
			ClockImage.renderBevel();
				
		clock_pix = ClockImage.convertToPixmap(wm->getDisplay(), DefaultVisual(wm->getDisplay(), wm->screen), clock.window);	

	}
}

void Toolbar::reconfigure()
{
	createToolBarImage();
	createButtonImage();
	createClockImage();

	XSetWindowBackgroundPixmap(wm->getDisplay(), frame.window, frame_pix);
	XSetWindowBackgroundPixmap(wm->getDisplay(), clock.window, clock_pix);
	XSetWindowBackgroundPixmap(wm->getDisplay(), close.window, button_pix);
	XSetWindowBackgroundPixmap(wm->getDisplay(), cycleUp.window, button_pix);
	XSetWindowBackgroundPixmap(wm->getDisplay(), cycleDown.window, button_pix);
	
	XClearWindow(wm->getDisplay(), frame.window);
	XClearWindow(wm->getDisplay(), clock.window);
	XClearWindow(wm->getDisplay(), cycleUp.window);		
	XClearWindow(wm->getDisplay(), cycleDown.window);		

	// check to see if toolbar is to be at top or bottom of the screen
  	// then place toolbar in correct location
	if (wm->theme->getToolbarLocationTop())
	{
  		frame.width = wm->getXRes() - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = 0;
		
		XMoveResizeWindow(wm->getDisplay(), frame.window, frame.x, frame.y , frame.width, frame.height);
  	} else {
  		frame.width = wm->getXRes() - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = wm->getYRes() - frame.height;
		
		XMoveResizeWindow(wm->getDisplay(), frame.window, frame.x, frame.y , frame.width, frame.height);
	}

	checkClock(True);

}

void Toolbar::do_button_press_event(XButtonEvent *e)
{	
	wm->Grab();

	switch (e->button) {
			
		case Button1:
		if(close.window == e->window)
		{
			XSetWindowBorderWidth(wm->getDisplay(), close.window, 2);
		} else if(cycleUp.window == e->window)
		{
			XSetWindowBorderWidth(wm->getDisplay(), cycleUp.window, 2);
		} else if(cycleDown.window == e->window)
		{
			XSetWindowBorderWidth(wm->getDisplay(), cycleDown.window, 2);
		}
		break;
	
		case Button2:
		{ 
		}
			
		break;
				
		case Button3:
		{
		}
		break;
	}
	
	wm->Ungrab();
}

void Toolbar::do_button_release_event(XButtonEvent *e)
{
	switch (e->button) {
			
		case Button1:
			if(close.window == e->window)
			{
				XSetWindowBorderWidth(wm->getDisplay(), close.window, 1);
				wm->quit_nicely();
			} else if(cycleUp.window == e->window)
			{
				XSetWindowBorderWidth(wm->getDisplay(), cycleUp.window, 1);
				XCirculateSubwindowsUp(wm->getDisplay(), RootWindow(wm->getDisplay(), wm->screen));
			} else if(cycleDown.window == e->window)
			{
				XSetWindowBorderWidth(wm->getDisplay(), cycleDown.window, 1);
				XCirculateSubwindowsDown(wm->getDisplay(), RootWindow(wm->getDisplay(), wm->screen));
			}
		break;
	
		case Button2:
		{ 
		}
			
		break;
				
		case Button3:
		{
		}
		break;
	}
}

void Toolbar::redraw(bool state)
{
	XClearWindow(wm->getDisplay(), cycleUp.window);
	XClearWindow(wm->getDisplay(), cycleDown.window);
	XClearWindow(wm->getDisplay(), close.window);

	XDrawLine(wm->getDisplay(), close.window, wm->border_gc, 2,2,close.width - 3,close.height-3);
	XDrawLine(wm->getDisplay(), close.window, wm->border_gc, 2,close.height - 3,close.width-3,2);

	int hh = cycleUp.height / 2, hw = cycleUp.width / 2;

	XPoint cycleDownpts[3];
	cycleDownpts[0].x = hw + 3; 
	cycleDownpts[0].y = hh -2;
	cycleDownpts[1].x = -3; 
	cycleDownpts[1].y = 6;
	cycleDownpts[2].x = -3; 
	cycleDownpts[2].y = -6;
		
		
	XFillPolygon(wm->getDisplay(), cycleDown.window, wm->border_gc, cycleDownpts, 3, Convex,
	       CoordModePrevious);	
	

	XPoint cycleUppts[3];
	cycleUppts[0].x = hw - 4; 
	cycleUppts[0].y = hh + 4;
	cycleUppts[1].x = 7; 
	cycleUppts[1].y = 0;
	cycleUppts[2].x = -3; 
	cycleUppts[2].y = -7;
    
	XFillPolygon(wm->getDisplay(), cycleUp.window, wm->border_gc, cycleUppts, 3, Convex,
	       CoordModePrevious);		
	
	XSetWindowBackgroundPixmap(wm->getDisplay(), cycleUp.window, button_pix);
	XSetWindowBackgroundPixmap(wm->getDisplay(), cycleDown.window, button_pix);
	XSetWindowBackgroundPixmap(wm->getDisplay(), close.window, button_pix);
	XSetWindowBackgroundPixmap(wm->getDisplay(), frame.window, frame_pix);
	
	checkClock(True);
}

void Toolbar::handle_expose_event(XExposeEvent *e)
{
	if(e->count == 0) redraw(False);
}

void Toolbar::checkClock(bool redraw)
{
  static int hour, minute, day, month, year;
  time_t tmp;
  struct tm *tt;
  
  if ((tmp = time(NULL)) != -1) {
    tt = localtime(&tmp);
    if (tt->tm_min != minute || tt->tm_hour != hour || day != tt->tm_mday ||
    	month != tt->tm_mon || year != tt->tm_year) {

      hour = tt->tm_hour;
      minute = tt->tm_min;
      day = tt->tm_mday;
      month = tt->tm_mon;
      year = tt->tm_year;
      XClearWindow(wm->getDisplay(), clock.window);
      redraw = True;
    }
  }
  
  if (redraw) {
	char t[18];
	sprintf(t, "%02d:%02d%cm %02d %s %02d",
            ((hour > 12) ? hour - 12 : ((hour == 0) ? 12 : hour)), minute,
	    ((hour >= 12) ? 'p' : 'a'), day, month_abbreviation[month], year+1900);
	  
	int len = strlen(t);
	int font_width_test = XTextWidth(wm->font, t, strlen(t)) + 10;
	int x_pos=0;
	
	if (font_width < font_width_test)
	{
		font_width = font_width_test;
		x_pos =  wm->getXRes() - font_width - 20;

		XMoveResizeWindow(wm->getDisplay(), clock.window, x_pos , clock.y , font_width,  15);
	}
	    
	XDrawString(wm->getDisplay(), clock.window, wm->string_gc,
		5, wm->font->ascent, t, len); 
  }
}

void Toolbar::hideToolbar()
{
	XUnmapSubwindows(wm->getDisplay(), frame.window);
}

void Toolbar::showToolbar()
{
	XMapSubwindows(wm->getDisplay(), frame.window);
	XMapWindow(wm->getDisplay(), frame.window);
}
