// wm.cc
//
// Frank Hale 
// frankhale@yahoo.com
#include "windowmanager.hh"

WindowManager *wm;

WindowManager::WindowManager(int argc, char **argv) 
{
	::wm = this;
	
	user_specified_display = "";
	user_specified_font = DEFAULT_FONT;
	user_specified_foreground_color = DEFAULT_FOREGROUND_COLOR;
	user_specified_background_color = DEFAULT_BACKGROUND_COLOR;
	user_specified_border_color = DEFAULT_BORDER_COLOR;
	
	head_client = new Client();
	
	ClientList = new LinkedList<Client>;
	menulist = new LinkedList<BaseMenu>;

	for (int i = 1; i < argc; i++) {
		if ((strcmp(argv[i], "-fn")==0) && i+1<argc)
			user_specified_font = argv[++i];
		else if ((strcmp(argv[i], "-display")==0) && i+1<argc)
			user_specified_display = argv[++i];
		else if ((strcmp(argv[i], "-fg")==0) && i+1<argc)
			user_specified_foreground_color = argv[++i];
		else if ((strcmp(argv[i], "-bg")==0) && i+1<argc)
			user_specified_background_color = argv[++i];
		else if ((strcmp(argv[i], "-bd")==0) && i+1<argc)
			user_specified_border_color = argv[++i];
		else if ((strcmp(argv[i], "-version")==0) && i<argc) {
			printf("version %s\nby: %s\nemail: %s\n", windowmanager_version, windowmanager_author, windowmanager_author_email);
			exit(0);
		}
		else {
			printf("usage: %s %s", windowmanager_name, windowmanager_usage);
			exit(2);
		}
	}

	theme = new Theme();
	
	setup_display();
	scan_windows();
	do_event_loop();
	exit(1);
}

WindowManager::~WindowManager() 
{
	XFreeFont(dpy, font);
	
	XFreeGC(dpy, invert_gc);
	XFreeGC(dpy, string_gc);
	XFreeGC(dpy, border_gc);
	
	if(ClientList->count())
	{
		LinkedListIterator<Client> c(ClientList);
		for(; c.current(); c++) 
		{
			ClientList->remove(c.current());
		}
	}
	
	if(menulist->count())
	{
		LinkedListIterator<BaseMenu> m(menulist);
		for(; m.current(); m++) 
		{
			menulist->remove(m.current());
		}
	}
	
	delete theme;
	delete t;
	delete rm;
	delete head_client;
	delete ClientList;
	delete menulist;
}

#ifdef SHAPE
int WindowManager::QueryShapeExtentions()
{
	return shape = XShapeQueryExtension(dpy, &shape_event, &dummy2);
}
#endif

void WindowManager::setup_display()
{
	XGCValues gv;
	XSetWindowAttributes attr;
	XColor dummy;

#ifdef SHAPE
	int dummy2;
#endif

	XSetErrorHandler(handle_xerror);
	signal(SIGTERM, (void (*)(int)) signalhandler);
	signal(SIGINT,  (void (*)(int)) signalhandler);
	signal(SIGHUP,  (void (*)(int)) signalhandler);

	dpy = XOpenDisplay(user_specified_display);

	if (!dpy) { 
		err("can't open display %s", user_specified_display);
		exit(1);
	}

	screen = DefaultScreen(dpy);
	root = RootWindow(dpy, screen);
	
	XDefineCursor(dpy, root, XCreateFontCursor(wm->dpy, XC_left_ptr));

	// Atoms			
	xa_wm_state        = XInternAtom(dpy, "WM_STATE", False);
	xa_wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
	xa_wm_protos       = XInternAtom(dpy, "WM_PROTOCOLS", False);
	xa_wm_delete       = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
	xa_wm_cmapwins     = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);
	xa_wm_take_focus   = XInternAtom(dpy, "WM_TAKE_FOCUS", False);
	xa_wm_motif_hints  = XInternAtom(dpy, "_MOTIF_WM_HINTS", False);
	
	// Colors
	XAllocNamedColor(dpy, DefaultColormap(dpy, screen), user_specified_foreground_color, &foreground_color, &dummy);
	XAllocNamedColor(dpy, DefaultColormap(dpy, screen), user_specified_background_color, &background_color, &dummy);
	XAllocNamedColor(dpy, DefaultColormap(dpy, screen), user_specified_border_color, &border_color, &dummy);
	
	font = XLoadQueryFont(dpy, user_specified_font);
	if (!font) font = XLoadQueryFont(dpy, DEFAULT_FONT);
	iheight = font->ascent + font->descent + 2*DEFAULT_SPACE;

	#ifdef SHAPE
		QueryShapeExtentions();
	#endif
	
	move_curs = XCreateFontCursor(dpy, XC_fleur);
	resize_curs = XCreateFontCursor(dpy, XC_plus);
	left_arrow = XCreateFontCursor(dpy, XC_left_ptr);

	gv.function = GXcopy;
	gv.foreground = foreground_color.pixel;
	gv.font = font->fid;
	string_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCFont, &gv);

	gv.foreground = border_color.pixel;
	gv.line_width = DEFAULT_BORDER_SIZE;
	border_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCLineWidth, &gv);

	gv.function = GXinvert;
	gv.subwindow_mode = IncludeInferiors;
	invert_gc = XCreateGC(dpy, root, GCFunction | GCSubwindowMode | GCLineWidth, &gv);

	attr.event_mask = SubstructureRedirectMask | SubstructureNotifyMask | ColormapChangeMask | 
	                  ButtonPressMask | ButtonReleaseMask | PropertyChangeMask | PointerMotionMask | FocusChangeMask |
			  ExposureMask;
	XChangeWindowAttributes(dpy, root, CWEventMask, &attr);
	
	xres = WidthOfScreen(ScreenOfDisplay(wm->dpy, wm->screen));
	yres = HeightOfScreen(ScreenOfDisplay(wm->dpy, wm->screen));
		
	#ifdef GNOME
		initGnome();
		reportLevelOfGNOMECompliance();
		setupGnomeButtonProxy();
	#endif
	
	t = new Toolbar();
	rm = new RootMenu();
	rm->parseMenuConfigurationFile();
}

void WindowManager::scan_windows()
{
	unsigned int i, nwins;
	Window dw1, dw2, *wins;
	XWindowAttributes attr;

	XQueryTree(dpy, root, &dw1, &dw2, &wins, &nwins);
	for (i = 0; i < nwins; i++) {
		XGetWindowAttributes(dpy, wins[i], &attr);
		if (!attr.override_redirect && attr.map_state == IsViewable)
			head_client->make_new_client(wins[i]);
	}
	XFree(wins);
}

void WindowManager::do_event_loop()
{
	XEvent ev;

	BaseMenu *m = None;

	int xfd = ConnectionNumber(wm->dpy);

	for (;;) {

		if(XPending(wm->dpy))
		{
			XNextEvent(wm->dpy, &ev);

			switch (ev.type) {
				case ButtonPress:
				{
					head_client->handle_button_press_event(&ev.xbutton); 
					t->do_button_press_event(&ev.xbutton);
					
					if(m = findInMenuList(&ev.xbutton.window))
					{
						m->handle_button_press_event(&ev.xbutton);	
					}
				
					switch(ev.xbutton.button)
					{
				
						case 1: {
							#ifdef GNOME
								proxyGnomeButtonPressEvents(&ev); 
							#endif
						break;
						}
					
						case 2: {
							#ifdef GNOME
								proxyGnomeButtonPressEvents(&ev);
							#endif	
						break;
						}

						case 3: {
					 		if(ev.xbutton.window == root)
					  		{
								rm->show();
					  		}
						break;	
						}
						
						default:
						{
							#ifdef GNOME
								proxyGnomeButtonPressEvents(&ev);
							#endif
						}
						break;

					}
				break;
				}
			
				case ButtonRelease:
				{
					head_client->handle_button_release_event(&ev.xbutton); 
					t->do_button_release_event(&ev.xbutton);

					if(m = findInMenuList(&ev.xbutton.window))
					{
						m->handle_button_release_event(&ev.xbutton);	
					}
										
					switch(ev.xbutton.button)
					{
					
						case 1:
						case 2: 
						case 3: 
						default:
							#ifdef GNOME
								proxyGnomeButtonReleaseEvents(&ev);
							#endif
						break;
					}
				break;
				}
				
				case ConfigureRequest:
				{
					head_client->handle_configure_request(&ev.xconfigurerequest); 
				
				break;
				}
				
				case ConfigureNotify:
				{
				break;
				}
				
				case DestroyNotify:
				{
					head_client->handle_destroy_notify(&ev.xdestroywindow);
				break;
				}
				
				case FocusIn:
				{
				
					head_client->handle_focus_in_event(&ev.xfocus);
				break;
				}
				
				case FocusOut:
				{
				
					head_client->handle_focus_out_event(&ev.xfocus);

				break;
				}
				
				case MapRequest:
				{
					head_client->handle_map_request(&ev.xmaprequest);
				
				break;
				}
			
				case UnmapNotify:
				{
					head_client->handle_unmap_event(&ev.xunmap); 
				break;
				}
			
				case ClientMessage:
				{
					head_client->handle_client_message(&ev.xclient); 
				break;
				}
			
				case ColormapNotify:
				{
					head_client->handle_colormap_change(&ev.xcolormap); 
				break;
				}
			
				case PropertyNotify:
				{
					head_client->handle_property_change(&ev.xproperty); 
				break;
				}

				case LeaveNotify:
				{									
					if(m = findInMenuList(&ev.xcrossing.window))
					{
						m->handle_leave_notify(&ev.xcrossing);	
					}

				break;
				}
			
				case EnterNotify:
				{
					head_client->handle_enter_event(&ev.xcrossing); 
										
					if(m = findInMenuList(&ev.xcrossing.window))
					{
						m->handle_enter_notify(&ev.xcrossing);	
					}
				break;
				}

				case MotionNotify:
				{
					head_client->handle_motion_event(&ev.xmotion); 
					
					if(m = findInMenuList(&ev.xmotion.window))
					{
						m->handle_motion_notify_event(&ev.xmotion);
					}
				break;
				}
							
				case Expose:
				{
					head_client->handle_expose_event(&ev.xexpose);
				
					rm->handle_expose_event(&ev.xexpose); 
					t->handle_expose_event(&ev.xexpose); 
					
					if(m = findInMenuList(&ev.xexpose.window))
					{
						m->handle_expose_event(&ev.xexpose); 
					}
				
					break;
				}
			
				case ReparentNotify:
				{
					head_client->handle_reparent_event(&ev.xreparent); 
				
				break;
				}

				default: 
				{				
					#ifdef SHAPE
					if (wm->shape && ev.type == wm->shape_event) 
					head_client->handle_shape_change((XShapeEvent *)&ev);
					#endif
					 	 			
				break;		 
				}	
			
			
			}
		
		} else {
			
			fd_set rfds;
			FD_ZERO(&rfds);
			FD_SET(xfd, &rfds);
      
			struct timeval tv;
			tv.tv_sec = 0;
			tv.tv_usec = 1000;
      
			select(xfd + 1, &rfds, 0, 0, &tv);
			t->checkClock();
		}

	}
	

}

// Function:
//
// void WindowManager::quit_nicely()
//
// Purpose:
//
// This functions sole purpose in life is to preform clean up so that we can
// shutdown Sapphire safely.
//
// Returns:
// 
// This function returns nothing.
//
void WindowManager::quit_nicely()
{
	XFreeFont(dpy, font);

	XSetInputFocus(dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
	XInstallColormap(dpy, DefaultColormap(dpy, screen));
	XCloseDisplay(dpy);
	
	printf("%s", windowmanager_exit_message);
	
	exit(0);
}

// Function:
//
// void WindowManager::start_another_windowmanager(char *windowmanager)
//
// Purpose:
//
// This functions takes care of the necessary shutdown process of Sapphire
// and starts another window manager which is passed through char* windowmanager
// which is the name of the filename for the window manager which this function
// will start up.
// 
// Returns:
// 
// This function returns nothing.
//
void WindowManager::start_another_windowmanager(char *windowmanager)
{
	//XFreeFont(dpy, font);
	//XSetInputFocus(dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
	//XInstallColormap(dpy, DefaultColormap(dpy, screen));
	
	// Here we will do the necessary steps in order to load new window manager.

	// Not finished......
}

BaseMenu* WindowManager::findInMenuList(Window *w)
{
	if(menulist->count())
	{
		LinkedListIterator<BaseMenu> m(menulist);
				
		for(; m.current(); m++) 
		{
			if(*w == m.current()->getMenuWindow()      || 
			   *w == m.current()->getMenuTitleWindow() ||
			   *w == m.current()->getMenuItemWindow())
			{
				return m.current();
			}
		}
	}
	
	return NULL;
}

#ifdef GNOME

// http://developer.gnome.org/doc/standards/wm/c4.html#AEN6
//
// The following was taken from the URL above:
//
// There is a single unambiguous way to detect if there currently is a GNOME compliant Window Manager 
// running. It is the job of the Window Manager to set up a few things to make this possible. Using 
// the following method it is also possible for applications to detect compliance by receiving an event 
// when the Window Manager exits.
//
// To do this the Window Manager should create a Window, that is a child of the root window. There is 
// no need to map it, just create it. The Window Manager may reuse ANY window it has for this purpose - 
// even if it is mapped, just as long as the window is never destroyed while the Window Manager is running.
//
// Once the Window is created the Window Manager should set a property on the root window of the name 
// _WIN_SUPPORTING_WM_CHECK, and type CARDINAL. The atom's data would be a CARDINAL that is the Window ID
//  of the window that was created above. The window that was created would ALSO have this property set 
// on it with the same values and type.
void WindowManager::initGnome()
{
   Atom                atom_set;
   CARD32              val;
         
   atom_set = XInternAtom(dpy, "_WIN_SUPPORTING_WM_CHECK", False);
   gnome_win = XCreateSimpleWindow(dpy, root, -200, -200, 5, 5, 0, 0, 0);
   val = gnome_win;
   XChangeProperty(dpy, root, atom_set, XA_CARDINAL, 32, 
                   PropModeReplace, (unsigned char *) &(val), 1);
   XChangeProperty(dpy, gnome_win, atom_set, XA_CARDINAL, 32, PropModeReplace,
                   (unsigned char *) &(val), 1);
}

// http://developer.gnome.org/doc/standards/wm/x13.html
//
// The following was taken from the URL above:
//
// It is important to list which parts of GNOME Window Manager compliance are supported. This is done 
// fairly easily by doing the following:
//
// Create a property on the root window of the atom name _WIN_PROTOCOLS. This property contains a 
// list(array)of atoms that are all the properties the Window Manager supports. These atoms are any 
// number of the following:
//
//    _WIN_LAYER 
//    _WIN_STATE 
//    _WIN_HINTS 
//    _WIN_APP_STATE 
//    _WIN_EXPANDED_SIZE 
//    _WIN_ICONS 
//    _WIN_WORKSPACE 
//    _WIN_WORKSPACE_COUNT 
//    _WIN_WORKSPACE_NAMES 
//    _WIN_CLIENT_LIST
//
// If you list one of these properties then you support it and applications can expect information 
// provided by, or accepted by the Window Manager to work.
//
void WindowManager::reportLevelOfGNOMECompliance()
{
  	Window              root_window;
  	Atom                atom_set;
  	Atom                list[10];

  	atom_set = XInternAtom(dpy, "_WIN_PROTOCOLS", False);
  	list[0] = XInternAtom(dpy, "_WIN_LAYER", False);
  	list[1] = XInternAtom(dpy, "_WIN_STATE", False);
  	list[2] = XInternAtom(dpy, "_WIN_HINTS", False);
	list[3] = XInternAtom(dpy, "_WIN_APP_STATE", False);
	list[4] = XInternAtom(dpy, "_WIN_EXPANDED_SIZE", False);
	list[5] = XInternAtom(dpy, "_WIN_ICONS", False);
	list[6] = XInternAtom(dpy, "_WIN_WORKSPACE", False);
	list[7] = XInternAtom(dpy, "_WIN_WORKSPACE_COUNT", False);
	list[8] = XInternAtom(dpy, "_WIN_WORKSPACE_NAMES", False);
	list[9] = XInternAtom(dpy, "_WIN_CLIENT_LIST", False);
   
	XChangeProperty(dpy, root, atom_set, XA_ATOM, 32, PropModeReplace,
                   (unsigned char *)list, 10);
}

// http://developer.gnome.org/doc/standards/wm/c71.html
// 
// The next 3 functions are included because: (taken from the URL above)
//
// X imposes a limitiation - that only 1 client can select for button presses on a window - 
// this is due to the implicit grab nature of button press events in X. This
// poses a problem when more than one client wishes to select for these events on the same window - 
// ie the root window, or in the case of a WM that has more than one root window (virtual root windows)
// any of these windows. The solution to this is to have the client that recieves these events handle any 
// of the events it is interested in, and then ``proxy'' or ``pass on'' any events it doesnt not care about. 
// Seeing the traditional model has always been that the WM selects for butotn presses on the desktop, 
// it is only natural that it keep doing this BUT have a way of sending unwanted presses onto some 
// other process(es) that may well be interested.
//
void WindowManager::setupGnomeButtonProxy()
{
	Atom atom_set; 
	CARD32 val; 
	
	atom_set = XInternAtom(dpy, "_WIN_DESKTOP_BUTTON_PROXY",False); 
	bpress_win = XCreateSimpleWindow(dpy, root, -80, -80, 24, 24, 0, 0, 0); 
	
	val = bpress_win; 
	
	XChangeProperty(dpy, root, atom_set, XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &(val), 1); 
	XChangeProperty(dpy, bpress_win, atom_set, XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &(val), 1); 
}

void WindowManager::proxyGnomeButtonPressEvents(XEvent *e)
{
 	XUngrabPointer(dpy, CurrentTime);
 	XSendEvent(dpy, bpress_win, False, SubstructureNotifyMask, e);
}

void WindowManager::proxyGnomeButtonReleaseEvents(XEvent *e)
{
 	XSendEvent(dpy, bpress_win, False, SubstructureNotifyMask, e);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif

void WindowManager::grabPointer(Window w, unsigned int mask, Cursor curs)
{
	if (! XGrabPointer(dpy, w, False, mask, GrabModeAsync, GrabModeAsync, None, curs, CurrentTime) == GrabSuccess ) 
		return;	
}

void WindowManager::ungrabPointer()
{
	XUngrabPointer(dpy, CurrentTime);	
}
