// File: wm.hh
//
// Description:
//
// This file declares the class WindowManager. This is the core class which controls the event
// loop for the window manager.
//
//
// Frank Hale 
// frankhale@yahoo.com
//
// Updated 20 Jan 2000
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _WM_HH_
#define _WM_HH_

#include "sapphire.hh"

class WindowManager {

private: /* variables */
	Toolbar *t;

	LinkedList<BaseMenu> *menulist;
	LinkedList<Client> *ClientList;

	char *user_specified_display;
	char *user_specified_font;
	char *user_specified_foreground_color;
	char *user_specified_background_color;
	char *user_specified_border_color;


	Atom		xa_wm_state;
	Atom		xa_wm_change_state;
	Atom		xa_wm_protos;
	Atom		xa_wm_delete;
	Atom		xa_wm_cmapwins;
	Atom		xa_wm_take_focus;
	Atom		xa_wm_motif_hints;

	Window		root;
	Display		*dpy;

	// Gnome stuff
	#ifdef GNOME
		Window          bpress_win;
		Window		gnome_win;
	#endif
	
	RootMenu 	*rm;
	
	Client		*head_client;	
	
	#ifdef SHAPE
		Bool		shape;
		int		shape_event;
	#endif

	int	xres;
	int 	yres;


public:	/* variables */

	
	// Most of the variables that follow shouldn't be public and should be private.
	// These should only be accessible through get / set functions. External 
	// classes should be allowed to manipulate certain variables directly. 
	//
	// I will add this in later.....
	
	Theme 		*theme;
	
	int		screen;
	
	// Graphics contexts
	GC		invert_gc;
	GC		string_gc;
	GC		border_gc;
	
	int		iheight;
	
	XColor		foreground_color;
	XColor		background_color; 
	XColor		border_color;
		
	XFontStruct	*font;	
	
	Cursor		move_curs;
	Cursor		resize_curs;
	Cursor		left_arrow;
	

public: /* member functions */
	
	WindowManager(int argc, char **argv);
	~WindowManager();

	// Client List member functions
	void insertClient(Client *c) { ClientList->insert(c); }
	void removeClient(Client *c) { ClientList->remove(c); }
	unsigned int getClientCount() const { return ClientList->count(); }
	LinkedList<Client>* getClientListObject() const { return ClientList; }

	// Server Grab member functions	
	void Grab() { XGrabServer(dpy); }
	void Ungrab() { XUngrabServer(dpy); }

	void addToMenuList(BaseMenu *m) { menulist->insert(m); }		
	BaseMenu* findInMenuList(Window *w);
		
  	Atom getWMChangeStateAtom() const { return xa_wm_change_state; }
  	Atom getWMStateAtom() const       { return xa_wm_state; }
  	Atom getWMDeleteAtom() const      { return xa_wm_delete; }
 	Atom getWMProtocolsAtom() const   { return xa_wm_protos; }
  	Atom getWMFocusAtom() const       { return xa_wm_take_focus; }
  	Atom getWMColormapAtom() const    { return xa_wm_cmapwins; }
	Atom getMotifWMHintsAtom() const      { return xa_wm_motif_hints; }

	Display* getDisplay() const { return dpy; }
	Window  getRootWindow() const { return root; }

	RootMenu* getRootMenu() const { return rm; }

	void grabPointer(Window w, unsigned int mask, Cursor curs);
	void ungrabPointer();

	void quit_nicely();

	void start_another_windowmanager(char *windowmanager);
	
	int getXRes() const { return xres; }
	int getYRes() const { return yres; }
		
private: /* member functions */
	
	void setup_display();
	void scan_windows();
	void do_event_loop();
	
	#ifdef GNOME
		void initGnome();
		void reportLevelOfGNOMECompliance();
		void setupGnomeButtonProxy();
		void proxyGnomeButtonPressEvents(XEvent *e);
		void proxyGnomeButtonReleaseEvents(XEvent *e);
		void setupGnomeClientList();
	#endif
	
	#ifdef SHAPE
		int QueryShapeExtentions();
	#endif
};

extern WindowManager *wm;

#endif
